%%%    SWI Prolog Grammar for Argument Patterns

%% Files to be loaded and predicates for using the grammar

:- ensure_loaded('regular_expressions.pro'). % + = (1..10), * = (0..10)
:- ensure_loaded('prepositions.pro').

check_pattern(Pattern) :-              % is true, if Pattern is a wellformed 
	atom_chars(Pattern, CharList), % pattern according to the grammar 
%	writeq(CharList),              % given below
%	write('\n'),                   % syntax: 
	pattern(CharList,[]).          % ?- check_pattern('NP0:_ofNP2').

check_typed_pattern(Pattern) :-        % is true, if Pattern is a wellformed 
	atom_chars(Pattern, CharList), % typed pattern according to the grammar 
%	writeq(CharList),              % given below
%	write('\n'),                   % syntax: 
	typed_pattern(CharList,[]).    % ?- check_typed_pattern('NP0:_ofNP2').

patterns :-                 % writes all strings which are patterns according
	pattern(P,[]),      % to the pattern grammar given below
	concat_atom(P, String), 
	write(String),
	write('\n'),
	fail.


%% Begin of Pattern Grammar %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%% Note: The Grammar contains some comments on the meaning of the patterns
%%% which might not be understandable without having read 'pattern_grammar.pdf'

% 0. Separator

sep --> ['_'].   % _ separates constituents of patterns


% 1. Optionality tag

opt --> ['~'].   % I have chosen ~ instead of ?, because, under unix,
                 % ? cannot be used in file names.
                 % (One might want to store pattern XYZ in file XYZ.)

% 2. Head tag

head --> [':'].


% 3. Input-baseform-of-X-into-head-position tag

input --> ['-'].  % the [a-z]+ which follows input is the word whose
                  % baseform has to be introduced into the preceding cat

% 4. Syntactic categories

cat      --> r_AZ, r_azAZ_s.  % cat --> [A-Z][a-zA-Z]*
	                      % Note: [A-Z] = $i
	                      % Expressions of the form r_.* are
	                      % defined in the file 'regular_expressions.pro'

prep_cat --> prep, cat.	      % prep is defined in the file
                              % prepositions.pro
                              % prepositions may be empty

% 5. Argument numbers

argnum --> r_09. % argnum --> [0-9]  
                 % Note: r_09 = $j; the concatenation of $i and $j
                 % is the name of the function whose value has to be
	         % introduced into head_const
	
% 5. Pattern constituents

const --> prep_cat, argnum.                 % non-head-constituent

const --> prep_cat, argnum, opt.            % optional non-head-constituent

const --> cat, input, r_az_p.               % non-head-non-arg-constituent whose head position
                                            % has to be filled with (the baseform of) r_az_p.
                                            % (This is used for support verb constructions:
				            % NP1_VP-give_NP0:_aboutNP2 ; (lecture)n0 ==>
                                            % "The marvelous 1 will eventually give an interesting 
                                            % lecture about really weird 2s." ~ lecture(1,2).

head_const --> const, head.	            % head constituent

rest --> const.                             % series of constituents
rest --> const, sep, const.
rest --> const, sep, const, sep, const.
rest --> const, sep, const, sep, const, sep, const. % etc. (***)


% 6. Structure of patterns

pattern --> head_const, sep, rest.            % Any string that contains 
pattern --> rest, sep, head_const.            % one (and only one) head constituent
pattern --> rest, sep, head_const, sep, rest. % plus, in principle (cf. (***)), any
pattern --> head_const.                       % number of consts separated by seps
                                              % is a pattern.


% 7. Structure of typed patterns (= Patterns furnished with syntactic category)

typed_pattern --> left_bracket, pattern, right_bracket, r_AZ, r_azAZ09_s.
left_bracket --> ['('].
right_bracket --> [')'].


%% End of Pattern Grammar %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
